﻿using System;
using System.Globalization;
using System.Windows.Data;
using System.Windows.Markup;

namespace WpfInfragistics18Experiments
{
    public class AxisLabelConverter : MarkupExtension, IValueConverter
    {
          
        public bool UseRounding { get; set; }
        public bool UseAbbreviation { get; set; }

        public double ScaleMin { get; set; }
        public double ScaleMax { get; set; } 
        public double ScaleFactor { get; set; }

        public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
        {
            if (value.GetType() != typeof(double))
                return Binding.DoNothing;
             
            double v = (double)value; 

            // checking if we have all scale parameters
            if (double.IsNaN(ScaleMin) || double.IsNaN(ScaleMax) || double.IsNaN(ScaleFactor))
            {
                return v;
            }
            else
            {
                var acutal = ScaleMin + v * (ScaleMax - ScaleMin) / ScaleFactor;
                
                if (UseAbbreviation)
                {
                    if (acutal >= 1000000000)
                        return Math.Round(acutal / 1000000000, 1) + "B";
                    if (acutal >= 1000000)
                        return Math.Round(acutal / 1000000, 1) + "M";
                    if (acutal >= 1000)
                        return Math.Round(acutal / 1000, 1) + "K";

                    if (acutal >= 0.1)
                        return Math.Round(acutal, 1);
                    else
                        return Math.Round(acutal, 2);
                }
                else if (UseRounding)
                {
                    if (acutal >= 10)
                        return Math.Round(acutal / 100, 1) * 100;
                    else
                        return Math.Round(acutal, 1);

                }
                return acutal;
            }
        }
         
        public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// When implemented in a derived class, returns an object that is provided as the value of the target property for this markup extension. 
        /// </summary>
        /// <returns>
        /// The object value to set on the property where the extension is applied. 
        /// </returns>
        /// <param name="serviceProvider">A service provider helper that can provide services for the markup extension.</param>
        public override object ProvideValue(IServiceProvider serviceProvider)
        {
            return this;
        }
    }
}