﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Windows.Media;

namespace WpfInfragistics18Experiments
{
    public class PdfReader
    {
        private List<PdfPlotData> pdfs;
        
        private bool useTrueCoords = false;
        public List<PdfPlotData> Pdfs
        {
            get { return pdfs; }
            set { pdfs = value; }
        }

        public PdfReader()
        {
            pdfs = new List<PdfPlotData>();
        }

        public void ReadPdf(string filename )
        {
            Color[] colors = new Color[] {Colors.DarkRed, Colors.Green };
            using (var textReader = new StreamReader(filename))
            {
                var nText = textReader.ReadLine(); //N
                //TODO: parse for nText
                int nVals = 2;
                for (int i = 0; i < nVals; i++)
                {
                    //TODO: read weights
                    textReader.ReadLine();
                }
                var weights = new double[] {0.5, 0.5};

                var nAttrText = textReader.ReadLine();
                //TODO: parse for nAttributes
                int nAttributes = 2;
                string[] attrNames = new string[nAttributes];
                Range[] attrRanges = new Range[nAttributes];
                int[] nBins = new int[nAttributes];

                for (int i = 0; i < nAttributes; i++)
                {
                    //TODO: read attributes and bins
                    var line = textReader.ReadLine();
                    var split = line.Split(new string[] {"=", " "}, StringSplitOptions.RemoveEmptyEntries);
                    var name = split[0];
                    var min = double.Parse(split[1]);
                    var max = double.Parse(split[2]);
                    var nbin = int.Parse(split[3]);

                    if (i == 1)
                    {
                        min /= 1000;
                        max /= 1000;
                    }

                    attrNames[i] = name;
                    attrRanges[i] = new Range(min, max);
                    nBins[i] = nbin;
                }

                var xD = (attrRanges[0].Max - attrRanges[0].Min) / nBins[0];
                var yD = (attrRanges[1].Max - attrRanges[1].Min) / nBins[1];

                var xMin = attrRanges[0].Min;
                var yMin = attrRanges[1].Min;

                textReader.ReadLine(); //Useless line

                XRange = attrRanges[0];
                YRange = attrRanges[1];
                XBins = nBins[0];
                YBins = nBins[1];

                for (int k = 0; k < nVals; k++)
                {
                    PdfPlotData plotData = new PdfPlotData();
                    List<DataPoint> dataPoints = new List<DataPoint>();
                    plotData.Color = colors[k];

                    string name = string.Format("Series_{0}", k);

                    double[] xHist = new double[nBins[0]];
                    double[] yHist = new double[nBins[1]];

                    for (int i = 0; i < nBins[0]; i++)
                    {
                        for (int j = 0; j < nBins[1]; j++)
                        {
                            var val = textReader.ReadLine();
                            double d = double.Parse(val);
                            var x = xMin + i * xD;
                            var y = yMin + j * yD;

                            if (!useTrueCoords)
                            {
                                //With the bin numbers the contours are correct but I want to display on the axes the values corresponding to bins, not bin number
                                dataPoints.Add(new DataPoint(i, j, d));
                            }

                            else
                            {
                                //With these values the axes show the right values but the contours are not correct
                                dataPoints.Add(new DataPoint(x, y, d));
                            }
                            xHist[i] += d;
                            yHist[j] += d;
                        }
                    }

                    //Values for the x-axis and y-axis histograms
                    List<DataPoint> xHistPoints;
                    List<DataPoint> yHistPoints;
                    if (useTrueCoords)
                    {
                        //Using bin values
                        xHistPoints = xHist.Select((d, i) => new DataPoint(xMin + i * xD, 0, d / nBins[0] * weights[k])).ToList();
                        yHistPoints = yHist.Select((d, i) => new DataPoint(0, yMin + i * yD, d / nBins[1] * weights[k])).ToList();
                    }

                    else
                    {
                        //Using bin numbers
                        xHistPoints = xHist.Select((d, i) => new DataPoint(i, 0, d / nBins[0] * weights[k])).ToList();
                        yHistPoints = yHist.Select((d, i) => new DataPoint(0, i, d / nBins[1] * weights[k])).ToList();
                    }
                    pdfs.Add(plotData);
                    plotData.Points.Name = name;
                    plotData.Points.AddRange(dataPoints);

                    plotData.XHistogram.Clear();
                    plotData.XHistogram.Name = name;
                    plotData.XHistogram.AddRange(xHistPoints);
                    
                    plotData.YHistogram.Clear();
                    plotData.YHistogram.Name = name;
                    plotData.YHistogram.AddRange(yHistPoints);    
                }
            }
        }

        public Range YRange { get; set; }
        public Range XRange { get; set; }

        public int XBins { get; set; }
        public int YBins { get; set; }
    }
}